@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-#
 # SPDX-FileCopyrightText: 2014-2022 Fredrik Ahlberg, Angus Gratton,
 # Espressif Systems (Shanghai) CO LTD, other contributors as noted.
 #
@@ -30,7 +28,7 @@ __all__ = [
     "write_mem",
 ]
 
-__version__ = "4.4"
+__version__ = "4.5.1"
 
 import argparse
 import inspect
@@ -38,6 +36,7 @@ import os
 import shlex
 import sys
 import time
+import traceback
 
 from esptool.cmds import (
     chip_id,
@@ -64,12 +63,14 @@ from esptool.cmds import (
     write_flash_status,
     write_mem,
 )
+from esptool.config import load_config_file
 from esptool.loader import DEFAULT_CONNECT_ATTEMPTS, ESPLoader, list_ports
 from esptool.targets import CHIP_DEFS, CHIP_LIST, ESP32ROM
 from esptool.util import (
     FatalError,
     NotImplementedInROMError,
     flash_size_bytes,
+    strip_chip_name,
 )
 
 import serial
@@ -100,7 +101,7 @@ def main(argv=None, esp=None):
         "--chip",
         "-c",
         help="Target chip type",
-        type=lambda c: c.lower().replace("-", ""),  # support ESP32-S2, etc.
+        type=strip_chip_name,
         choices=["auto"] + CHIP_LIST,
         default=os.environ.get("ESPTOOL_CHIP", "auto"),
     )
@@ -469,6 +470,12 @@ def main(argv=None, esp=None):
         help="Change flash MMU page size.",
         choices=["64KB", "32KB", "16KB", "8KB"],
     )
+    parser_elf2image.add_argument(
+        "--pad-to-size",
+        help="The block size with which the final binary image after padding "
+        "must be aligned to. Value 0xFF is used for padding, similar to erase_flash",
+        default=None,
+    )
 
     add_spi_flash_subparsers(parser_elf2image, allow_keep=False, auto_detect=False)
 
@@ -623,6 +630,7 @@ def main(argv=None, esp=None):
 
     args = parser.parse_args(argv)
     print("esptool.py v%s" % __version__)
+    load_config_file(verbose=True)
 
     # operation function can take 1 arg (args), 2 args (esp, arg)
     # or be a member function of the ESPLoader class.
@@ -791,17 +799,15 @@ def main(argv=None, esp=None):
                         "Try checking the chip connections or removing "
                         "any other hardware connected to IOs."
                     )
-            except Exception as e:
-                esp.trace("Unable to verify flash chip connection ({}).".format(e))
+            except FatalError as e:
+                raise FatalError(f"Unable to verify flash chip connection ({e}).")
 
         # Check if XMC SPI flash chip booted-up successfully, fix if not
         if not esp.secure_download_mode:
             try:
                 flash_xmc_startup()
-            except Exception as e:
-                esp.trace(
-                    "Unable to perform XMC flash chip startup sequence ({}).".format(e)
-                )
+            except FatalError as e:
+                esp.trace(f"Unable to perform XMC flash chip startup sequence ({e}).")
 
         if hasattr(args, "flash_size"):
             print("Configuring flash size...")
@@ -1039,6 +1045,10 @@ def _main():
             "https://docs.espressif.com/projects/esptool/en/latest/troubleshooting.html"
         )
         sys.exit(1)
+    except StopIteration:
+        print(traceback.format_exc())
+        print("A fatal error occurred: The chip stopped responding.")
+        sys.exit(2)
 
 
 if __name__ == "__main__":
